from T2GEORES import formats as formats
import numpy as np
from datetime import datetime
import sys
import os
import pandas as pd
import sqlite3
import json

def converter(value,format_t2,def_value):
	"""Returns the input value on the specified format

	Parameters
	----------
	value : float-str
	  Parameter to be written
	format_t2 : dictionary
	  Contains the format and default values for every variable
	def_value : bool
	  True in case the default values is used

	Note
	----
	format_t2 comes from the file formats.py. In case a new type of section is included, the variable name needs to be specified on the file

	"""
	if 's' in format_t2:
		if value==None:
			eq_str_len=int(format_t2.replace(">","").replace("s",""))
			value=" "*eq_str_len
		else:
			value=str(value)
	elif 'E' in format_t2 and def_value:
		if value==None:
			eq_str_len=int(format_t2.split('.')[0].split('>')[1])
			value=" "*eq_str_len
			format_t2=">%ss"%eq_str_len
	elif 'd' in format_t2 and def_value:
		if value==None:
			eq_str_len=int(format_t2.split('d')[0].split('>')[1])
			value=" "*eq_str_len
			format_t2=">%ss"%eq_str_len
	output=format(value,format_t2)

	return output

def def_value_selector(section,key,input_dictionary,rocks=False):
	"""Evaluate if the necessary parameter has been defined on the input file

	Parameters
	----------
	section : str
	  Section from TOUGH2 file such as PARAMETER, RPCAP, etc.
	key : str
	  Parameter to evaluate
	input_dictionary : dictionary
	  Dictionary with the defined input information

	"""

	try:
		if not rocks:
			value=input_dictionary[section][key]
			def_value=False
		else:
			value=input_dictionary['ROCKS'][section][key]
			def_value=False
	except KeyError:
		if not rocks:
			value=formats.formats_t2[section][key][0]
			def_value=True
		else:
			value=formats.formats_t2['ROCKS'][key][0]
			def_value=True
	return value,def_value

def FCG_file_checker(name):
	"""Evaluate if the the file defining the FOFT, COFT or GOFT section exits or if it contains more than 100 lines

	Parameters
	----------
	name : str
	  Name of the file to check

	Note
	----
	Print messages are expected

	Returns
	-------
	str
	  string : string containing 100 lines of the selected file
	bool
	  isfile : True if the file exits

	"""

	string=""
	if os.path.isfile("../model/t2/sources/%s"%name):
		with open("../model/t2/sources/%s"%name) as file:
			isfile=True
			for cnt,line in enumerate(file.readlines()[:]):
				if cnt+1<100:
					string+="%s"%line
				else:
					print("Values above E%s(100) will not be printed, %s is not included"%(name,line))
	else:
		print("Error message: the %s file on ../model/t2/sources/%s is not prepared"%(name,name))
		isfile=False
	string+="\n"	
	return string,isfile

def diff_seconds(ref,date):
	"""Return the difference in seconds

	Parameters
	----------
	ref : datetime
	  Model's reference time
	date : datetime
	  An arbitrary time

	Returns
	-------
	int
	  diff_seconds : difference in seconds

	"""

	if isinstance(date, datetime) and isinstance(ref, datetime):
		diff_seconds=(date-ref).total_seconds()
	else:
		sys.exit("ref or date values are not datetime type")
	return diff_seconds

def PARAM_writer(input_dictionary):
	"""Return the PARAM section

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with the defined parameter information

	Returns
	-------
	str
	  string : string containing PARAM section

	"""
	string="PARAM----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	params_levels=[1,2,3,4]
	for level in params_levels:
		for key in formats.formats_t2['PARAMETERS']:
			if formats.formats_t2['PARAMETERS'][key][2]==level:
				if key=='RE1' and 'DELTEN' in input_dictionary['PARAMETERS'] and input_dictionary['PARAMETERS']['DELTEN']<0 :
					for index, time_step in enumerate(input_dictionary['PARAMETERS']['DELTEN_LIST']):
						string+=converter(time_step,formats.formats_t2['DELTEN_LIST_FORMAT'][1],False)
						if (index+1)%8==0:
							string+='\n'
					string+='\n'

				value,def_value=def_value_selector('PARAMETERS',key,input_dictionary)

				if not isinstance(value,list):
					string+=converter(value,formats.formats_t2['PARAMETERS'][key][1],def_value)
				else:
					for cnt, var in enumerate(value):
						string+=converter(var,formats.formats_t2['PARAMETERS'][key][cnt][2],def_value)
		string+='\n'
	string+="\n"
	return string

def TIMES_writer(input_dictionary):
	"""Return the TIMES section

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with the defined TIMES information

	Returns
	-------
	str
	  string : string containing the TIMES section

	"""

	string="TIMES----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"

	if any(input_dictionary['TIMES']['TIMES_N']):
		len_times=len(input_dictionary['TIMES']['TIMES_N'])
		input_dictionary['TIMES']['ITI']=len_times
		input_dictionary['TIMES']['ITE']=len_times
		if len_times>100:
			sys.exit("ITI cannot be higher than 100, reduce the number of times")
	for key in formats.formats_t2['TIMES']:

		value,def_value=def_value_selector('TIMES',key,input_dictionary)

		if not isinstance(value,np.ndarray):
			string+=converter(value,formats.formats_t2['TIMES'][key][1],def_value)
		else:
			string+='\n'
			for index, var in enumerate(value):
				string+=converter(diff_seconds(input_dictionary['ref_date'],var),formats.formats_t2['TIMES']['TIMES_N'][1],False)
				if (index+1)%8==0:
					string+='\n'
	string+="\n\n"
	return string

def wells_track_blocks_to_FOFT(input_dictionary,wells):
	"""It creates the FOFT section base on the well track

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with the data base location information
	wells: list
	  Wells to include on the FOFT section

	Returns
	-------
	file
	  FOFT: on ../model/t2/sources
	  
	Attention
	---------
	This function could have more than 100 elements depending on the number of layers and wells defined. For TOUGH2 6.x it is preffereable to use well_sources_to_FOFT

	Examples
	--------
	>>> wells_track_blocks_to_FOFT(input_dictionary)
	"""

	conn=sqlite3.connect(input_dictionary['db_path'])
	c=conn.cursor()

	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY middle;",conn)

	string=""
	for name in sorted(wells):
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in sorted(data_layer['correlative'].values):
				string+="%s\n"%(n+data_block['blockcorr'].values[0])
	conn.close()
	file=open("../model/t2/sources/FOFT",'w')
	file.write(string)
	file.close()

def well_sources_to_FOFT(input_dictionary,wells):
	""""It creates the FOFT section base on the well feedzone position

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with the data base location information
	wells: list
	  Wells to include on the FOFT section

	Returns
	-------
	file
	  FOFT: on ../model/t2/sources

	Examples
	--------
	>>> well_sources_to_FOFT(input_dictionary)
	"""

	conn=sqlite3.connect(input_dictionary['db_path'])
	c=conn.cursor()

	string=""
	for name in sorted(wells):
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellsource WHERE well='%s';"%name,conn)
		string+="%s\n"%(data_block['blockcorr'].values[0])
	conn.close()
	file=open("../model/t2/sources/FOFT",'w')
	file.write(string)
	file.close()

def FOFT_writer(input_dictionary):
	""""It wraps up the section FOFT

	Returns
	-------
	str
	  output: contains the FOFT section

	"""
	string,isfile=FCG_file_checker('FOFT')
	if isfile:
		output="""FOFT ----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"""
		output+=string
	else:
		output=""
	return output

def COFT_writer(input_dictionary):
	""""It wraps up the section COFT

	Returns
	-------
	str
	  output: contains the COFT section

	"""
	string,isfile=FCG_file_checker('COFT')
	if isfile:
		output="""COFT ----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"""
		output+=string
	else:
		output=""
	return output

def source_to_GOFT(input_dictionary):
	""""It creates the GOFT section base on the well feedzone position nickname

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with the data base location information

	Returns
	-------
	file
	  GOFT: on ../model/t2/sources

	Examples
	--------
	>>> source_to_GOFT(input_dictionary)
	"""


	conn=sqlite3.connect(input_dictionary['db_path'])
	c=conn.cursor()

	data_blockcorr=pd.read_sql_query("SELECT source_nickname FROM t2wellsource WHERE source_nickname LIKE 'SRC%';",conn)

	string=""
	for n in sorted(data_blockcorr['source_nickname'].values):
		string+="%s\n"%(n)

	conn.close()
	file=open("../model/t2/sources/GOFT",'w')
	file.write(string)
	file.close()

def GOFT_writer(input_dictionary):
	""""It wraps up the section COFT

	Returns
	-------
	str
	  output: contains the COFT section

	"""
	string,isfile=FCG_file_checker('GOFT')
	if isfile:
		output="""GOFT ----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"""
		output+=string
	else:
		output=""
	return output

def SOLVR_writer(input_dictionary):
	""""It wraps up the section SOLVR

	Returns
	-------
	str
	  output: contains the SOLVR section


	"""
	string="SOLVR----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	
	cnt=0
	for key in formats.formats_t2['SOLVR']:
		value,def_value=def_value_selector('SOLVR',key,input_dictionary)
		if def_value: cnt+=1 
		string+=converter(value,formats.formats_t2['SOLVR'][key][1],def_value)
	string+="\n\n"

	if cnt==len(formats.formats_t2['SOLVR']):
		string=""
	return string

def GENER_adder(input_dictionary):
	""""It takes all the GENER section created depending on the type of run production or natural defined on the input dictionary

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary specficing the type of run

	Returns
	-------
	str
	  string : string containing the GENER section

	"""
	string="GENER----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	if os.path.isfile("../model/t2/sources/GENER_SOURCES") and input_dictionary['TYPE_RUN'] in ['natural','production']:
		with open("../model/t2/sources/GENER_SOURCES") as gener_source_file:
			for line in gener_source_file.readlines()[:]:
				string+="%s"%line
		if input_dictionary['TYPE_RUN']=='natural':
			string+="\n"
		elif input_dictionary['TYPE_RUN']=='production':
			if os.path.isfile("../model/t2/sources/GENER_PROD"):
				with open("../model/t2/sources/GENER_PROD") as gener_prod_file:
					for line in gener_prod_file.readlines()[:]:
						string+="%s"%line
			else:
				sys.exit("""Error message: the GENER_PROD file on ../model/t2/sources/GENER_PROD is not prepared\n
					        you might want to run write_gener_from_sqlite on t2gener.py file""")
			
			if os.path.isfile("../model/t2/sources/GENER_MAKEUP"):
				with open("../model/t2/sources/GENER_MAKEUP") as gener_prod_file:
					for line in gener_prod_file.readlines()[:]:
						string+="%s"%line
				string+="\n"
			else:
				print("""Error message: the GENER_MAKEUP file on ../model/t2/sources/GENER_MAKEUP is not prepared\n
					        you might want to run write_gener_from_sqlite on t2gener.py file""")
	else:
		sys.exit("""Error message: the GENER_SOURCES file on ..\model\t2\sources\GENER_SOURCES is not prepared\n
			        you might want to run write_geners_to_txt_and_sqlite on t2gener.py file or the type_run is not correct (natural or production)""")

	return string

def RPCAP_writer(input_dictionary):
	""""It wraps up the section RPCAP

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary containing the RPCAP information

	Returns
	-------
	str
	  output: contains the RPCAP section

	"""
	string="RPCAP----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	params_levels=[1,2]
	for level in params_levels:
		for key in formats.formats_t2['RPCAP']:
			if formats.formats_t2['RPCAP'][key][2]==level:
				value,def_value=def_value_selector('RPCAP',key,input_dictionary)
				string+=converter(value,formats.formats_t2['RPCAP'][key][1],def_value)
		string+='\n'
	string+="\n"
	return string

def MULTI_writer(input_dictionary):
	""""It wraps up the section MULTI

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary containing the MULTI information

	Returns
	-------
	str
	  output: contains the MULTI section

	"""
	string="MULTI----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	cnt=0
	for key in formats.formats_t2['MULTI']:
		value,def_value=def_value_selector('MULTI',key,input_dictionary)
		if def_value: cnt+=1 
		string+=converter(value,formats.formats_t2['MULTI'][key][1],def_value)
	string+="\n\n"

	if cnt==len(formats.formats_t2['MULTI']):
		string=""
	return string

def TITLE_writer(input_dictionary):
	""""It writes the TITLE of TOUGH2 file

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary containing the TITLE information

	Returns
	-------
	str
	  output: contains the TITLE section
	"""

	try:
		value=input_dictionary['TITLE']
	except KeyError:
		value=formats.formats_t2['TITLE'][0]
	return  value[0:80].ljust(80) +'\n'

def CONNE_from_steinar_to_t2():
	"""It takes the CONNE file from steinar and convertir into a CONNE TOUGH2 section (the vertical connections are affected by -1)

	Returns
	-------
	file
	  CONNE: on ../model/t2/sources/
	  
	Attention
	---------
	Comparing with steinar CONNE format the column width changes with the standard TOUGH2 input file

	Examples
	--------
	>>> CONNE_from_steinar_to_t2()
	"""
	source_file="../mesh/to_steinar/conne"
	output_file="../model/t2/sources/CONNE"

	string="CONNE----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	if os.path.isfile(source_file):
		with open(source_file) as rock_file:
			for line in rock_file.readlines()[1:]:
				linex=line.split()
				if float(linex[8])<=0:
					string+="%10s%20s%10.3E%10.3E%10.3E%10.2f\n"%(linex[0],int(linex[4]),float(linex[5]),float(linex[6]),float(linex[7]),float(linex[8]))
				else:
					string+="%10s%20s%10.3E%10.3E%10.3E%10.2f\n"%(linex[0],int(linex[4]),float(linex[5]),float(linex[6]),float(linex[7]),-1*float(linex[8]))
		string+='\n'
		file=open(output_file,'w')
		file.write(string)
		file.close()
	else:
		sys.exit("The file %s or directory do not exist"%source_file)

def merge_ELEME_and_in_to_t2():
	"""It takes the ELEME file from the steinar directory and convert it into a TOUGH2 format with coordinates 

	Returns
	-------
	file
	  ELEME: section from TOUGH2 file
	  
	Attention
	---------
	It adds the head of the section.

	Examples
	--------
	>>> merge_ELEME_and_in_to_t2()
	"""
	eleme_file="../mesh/to_steinar/eleme"
	output_file="../model/t2/sources/ELEME"


	block_eleme_file="../mesh/ELEME.json"
	if os.path.isfile(block_eleme_file):
		with open(block_eleme_file) as file:
			blocks=json.load(file)
	else:
		sys.exit("The file %s or directory do not exist"%block_eleme_file)

	col_eleme=[(0,6),(15,20),(20,30)]

	if os.path.isfile(eleme_file):
		string="ELEME----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
		data_eleme=pd.read_fwf(eleme_file,colspecs=col_eleme,skiprows=1,header=None,names=['ELEME','MA1','VOLX'])

		for i, row in data_eleme.iterrows():
			data_eleme.at[i,'VOLX'] = blocks[row['ELEME']]['VOLX']

		for i, row in data_eleme.sort_values(by=['VOLX'], ascending=False).iterrows():
			try:
				string+="%5s%10s%5s%10.3E%10s%10s%10.3E%10.3E%10.3E\n"%(row['ELEME'],\
					                                                    " ",\
					                                                row['MA1'],\
					                                                blocks[row['ELEME']]['VOLX'],\
		                                                                       " ",\
		                                                                       " ",\
		                                                            blocks[row['ELEME']]['X'],\
		                                                            blocks[row['ELEME']]['Y'],\
		                                                            blocks[row['ELEME']]['Z'])
			except KeyError:
				pass
		string+='\n'
		file=open(output_file,'w')
		file.write(string)
		file.close()
	else:
		sys.exit("The file %s or directory do not exist"%file_source_file)

def ATM_data():


	block_eleme_file="../mesh/vtu/ELEME.json"
	if os.path.isfile(block_eleme_file):
		with open(block_eleme_file) as file:
			blocks=json.load(file)
	else:
		sys.exit("The file %s or directory do not exist"%block_eleme_file)

	conne_file_st=open('../model/t2/sources/CONNE2','w')  
	conne_file=open('../model/t2/sources/CONNE','r')  
	data_conne=conne_file.readlines()

	for line in data_conne:
		#if both are on the conne line, the line should not exist
		if line[0:5] in blocks.keys() and line[5:10] in blocks.keys():
			conne_file_st.write(line)
		elif line[0:5] not in blocks.keys() and line[5:10] not in blocks.keys() :
			pass
		elif line[0:5] not in blocks.keys():
			if line[0]!=line[5]:
				conne_file_st.write('ATM01'+line[5:10]+'                   3 1.000e-02'+line[40:-1]+'\n')
			else:
				conne_file_st.write('ATM01'+line[5:10]+'                   1 1.000e-02'+line[40:-1]+'\n')
		elif  line[5:10] not in blocks.keys():
			if line[0]!=line[5]:
				conne_file_st.write(line[0:5]+'ATM01'+'                   3'+line[30:41]+'1.000e-02'+line[50:-1]+'\n')
			else:
				conne_file_st.write(line[0:5]+'ATM01'+'                   1'+line[30:41]+'1.000e-02'+line[50:-1]+'\n')

	conne_file_st.close()
	conne_file.close()


	eleme_file_st=open('../model/t2/sources/ELEME2','w')  
	eleme_file=open('../model/t2/sources/ELEME','r')  
	data_eleme=eleme_file.readlines()

	for line in data_eleme:
		if (line[0:5]) in blocks.keys():
			eleme_file_st.write(line)
		else:
			continue
	eleme_file_st.write("ATM01          ATMOS 1.000E+50\n")
	eleme_file_st.close()
	eleme_file.close()



def ELEME_adder(input_dictionary):
	""""It takes the ELEME file from ../model/t2/sources and print it as a string

	Returns
	-------
	str
	  string : string containing the ELEME section

	"""

	source_file="../model/t2/sources/ELEME"
	string=""
	if os.path.isfile(source_file):
		with open(source_file) as eleme_source_file:
			for line in eleme_source_file.readlines()[:]:
				string+="%s"%line
	else:
		sys.exit("The file %s or directory do not exist"%source_file)

	return string

def CONNE_adder(input_dictionary):
	""""It takes the CONNE file from ../model/t2/sources and print it as a string

	Returns
	-------
	str
	  string : string containing the CONNE section

	"""
	source_file="../model/t2/sources/CONNE"
	string=""
	if os.path.isfile(source_file):
		with open(source_file) as conne_source_file:
			for line in conne_source_file.readlines()[:]:
				string+="%s"%line
	else:
		sys.exit("The file %s or directory do not exist"%source_file)
	return string

def OUTPU_writer(input_dictionary):
	""""It writes the OUTPU of TOUGH2 file

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary containing the OUTPU information

	Returns
	-------
	str
	  output: contains the OUTPU section

	Examples
	--------
	'OUTPU':{'FORMAT':['CSV'],'ELEMENT_RELATED':{'ENTHALPY':[1],'SECONDARY':[1,5]},'CONNECTION_RELATED':{'FLOW':[1,2],},'GENER_RELATED':{'GENERATION':[None],'FRACTIONAL FLOW':[1]}}

	"""

	string="OUTPU----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"

	for outpu in input_dictionary['OUTPU']:
		if outpu=='FORMAT':
			string+="%s\n"%(input_dictionary['OUTPU'][outpu])
			n_variables=0
			string_temp=''	
		else:
			for key in input_dictionary['OUTPU'][outpu]:
				print(key)
				if outpu=='FORMAT':
					string+="%s\n"%(key)
					n_variables=0
					string_temp=''
				else:
					string_temp+=format(key,formats.formats_t2['OUTPU']['FORMAT'][0])
					n_variables+=1
					for i,parameter in enumerate(input_dictionary['OUTPU'][outpu][key]):
						try:
							if type(parameter)==formats.formats_t2['OUTPU'][outpu][key][i]:
								if parameter!=None:
									string_temp+=format(str(parameter),formats.formats_t2['OUTPU']['FORMAT'][1])
							else:
								sys.exit("OUTPU %s %s %s not correctly formatted"%(outpu,key,parameter))
						except KeyError:
							sys.exit("Key %s does not exist for OUTPU"%key)
    
					if not all(ft2==type(None) for ft2 in formats.formats_t2['OUTPU'][outpu][key]):
						cnt=0
						for value in formats.formats_t2['OUTPU'][outpu][key]:
							if value!=type(None):
								cnt+=1
						if i!=(cnt-1):
							sys.exit("For the key %s the number of arguments required are not enough"%key)
					string_temp+='\n'
	string=string+str(n_variables)+'\n'+string_temp+'\n'

	return string

def MOMOP_writer(input_dictionary):
	"""Return the MOMOP section

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with the defined MOMOP information

	Returns
	-------
	str
	  string : string containing MOMOP section

	"""
	string="MOMOP----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	string=''
	for key in formats.formats_t2['MOMOP']:
		if formats.formats_t2['MOMOP'][key][2]==1:
			value,def_value=def_value_selector('MOMOP',key,input_dictionary)
			string+=converter(value,formats.formats_t2['MOMOP'][key][1],def_value)
	string+="\n"
	return string

def ROCKS_writer(input_dictionary):
	"""Return the ROCKS section

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with the defined parameter information

	Returns
	-------
	str
	  string : string containing ROCKS section

	Examples
	--------
	Dictionary example::

		input_data={'ROCKS':{
				         'ROCK1':{
						         'MAT':'ROCK1',
						         'DROK':2.65E3,
						         'POR':0.1,
						         'CWET':2.1,
						         'SPHT':850,
						         'PER_X':1E-15,
						         'PER_Y':1E-15,
						         'PER_Z':1E-15,
				                  },
				        }
				}

	"""
	string="ROCKS----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	rocks_levels=[1,2,3,4]
	pass_valid=False
	for rock in input_dictionary['ROCKS']:
		for level in rocks_levels:
			break_line=False
			for key in formats.formats_t2['ROCKS']:
				if formats.formats_t2['ROCKS'][key][2]==level:
					if level==1:
						pass_valid=True
						break_line=True
					if 'NAD' in input_dictionary['ROCKS'][rock].keys():
						if input_dictionary['ROCKS'][rock]['NAD']>=1 and level==2:
							pass_valid=True
							break_line=True
						elif input_dictionary['ROCKS'][rock]['NAD']>=2 and level in [3,4]:
							pass_valid=True
							break_line=True
					if pass_valid:
						value,def_value=def_value_selector(rock,key,input_dictionary,rocks=True)
						string+=converter(value,formats.formats_t2['ROCKS'][key][1],def_value)
						pass_valid=False
			if break_line:
				string+='\n'
	string+="\n"
	return string

def t2_input(include_FOFT,include_SOLVR,include_COFT,include_GOFT,include_RPCAP,include_MULTI,include_START,include_TIMES,include_OUTPU,include_MOMOP,input_dictionary):
	""""It creates the FOFT section base on the well feedzone position

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary with all the necessary information to define a model
	include_FOFT: bool
	  If true includes the FOFT section on the TOUGH2 file
	include_SOLVR: bool
	  If true includes the SOLVR section on the TOUGH2 file
	include_COFT: bool
	  If true includes the COFT section on the TOUGH2 file
	include_GOFT: bool
	  If true includes the GOFT section on the TOUGH2 file
	include_RPCAP: bool
	  If true includes the RPCAP section on the TOUGH2 file
	include_MULTI: bool
	  If true includes the MULTI section on the TOUGH2 file
	include_START: bool
	  If true includes the START sentence is added on the top of PARAM section
	include_OUTPU: bool
	  If true includes the OUTPU section on the TOUGH2 file
	include_TIMES: bool
	  If true includes the TIMES section on the TOUGH2 file
	include_MOMOP: bool
	  If true includes the MOMOP section on the TOUGH2 file

	Returns
	-------
	file
	  t2: on ../model/t2/t2

	Examples
	--------
	>>> t2_input(input_dictionary,include_FOFT=True,include_SOLVR=True,include_COFT=False,include_GOFT=True,include_RPCAP=False,include_MULTI=True,include_START=True,include_MOMOP=True,include_OUTPU=True)
	"""

	secctions=[TITLE_writer,ROCKS_writer,PARAM_writer]

	if_functions_dictionary={'MOMOP':[include_MOMOP,MOMOP_writer],
							 'RPCAP':[include_RPCAP,RPCAP_writer],
							 'MULTI':[include_MULTI,MULTI_writer],
							 'FOFT':[include_FOFT,FOFT_writer],
							 'TIMES':[include_TIMES,TIMES_writer],
							 'SOLVR':[include_SOLVR,SOLVR_writer],
							 'COFT':[include_COFT,COFT_writer],
							 'GOFT':[include_GOFT,GOFT_writer],
							 'OUTPU':[include_OUTPU,OUTPU_writer]}

	for bools in if_functions_dictionary:
		if if_functions_dictionary[bools][0]:
			secctions.append(if_functions_dictionary[bools][1])

	secctions.extend([ELEME_adder,CONNE_adder])
	output=""
	for func in secctions:
		if func==PARAM_writer and include_START:
			output+="START----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
		output+=func(input_dictionary)

	#output+=GENER_adder(input_dictionary)

	output+="ENDCY----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"

	input_fi_file="../model/t2/t2"
	t2_file_out=open(input_fi_file, "w")
	t2_file_out.write(output)
	t2_file_out.close()


from T2GEORES import geometry as geomtr
from T2GEORES import formats as formats
import numpy as np
from iapws import IAPWS97
import pandas as pd
import os
from scipy import interpolate
import sys
import json
import math
import matplotlib.pyplot as plt

import seaborn as sns
import pylab as plb
import shapefile
from scipy.interpolate import griddata, interp1d
from scipy.stats import linregress

def patmfromelev(elev):
	"""It fines the atmospheric pressure base on an approximate formula
	
	Parameters
	----------
	elev
	  masl where the pressure is needed

	Returns
	-------
	float
	  Atmospheric pressure in bar

	Examples
	--------
	>>> patmfromelev(750)
	"""

	p_atm=(101325*((288+(-0.0065*elev))/288)**(-9.8/(-0.0065*287)))/100000
	return p_atm

def water_level_projection(MD,P,Tmin):
	"""It projects the water level based on a pressure log

	Parameters
	----------
	MD : array
	  Measure depth array
	P : array
	  Pressure array
	Tmin : float
	  Minimun temperature allowed

	Returns
	-------
	float
	  D2: water leverl projection
	float
	  Pmin: pressure at water level  

	Examples
	--------
	>>> water_level_projection(MD,P,120)
	"""

	#Creation of array with pressure
	P_array=np.linspace(min(P),max(P),4)
	P0=P_array[1]
	P1=P_array[2]

	#Pressure vs MD function
	MDfunc=interpolate.interp1d(P,MD)
	MD0=MDfunc(P0)
	MD1=MDfunc(P1)

	#water_level_assuming saturation temperature at surface
	delta_MD=MD0-MD1
	delta_P=P0-P1
	m=delta_MD/delta_P
	b=MD0-m*P0

	Pmin=IAPWS97(T=(Tmin+273.15),x=0).P*10

	D2=m*Pmin+b

	return D2,Pmin

def initial_conditions(input_dictionary,m,b,use_boiling=True,use_equation=False):
	"""Generates the tempearture and pressure initial conditions assuming boiling from top to bottom layer

	Parameters
	----------
	input_dictionary : dictionary
	  Contains the infomation of the layer under the keyword 'LAYER' and 'z_ref'. Also it contains the keyword 'INCONS_PARAM' with the specified initial conditions, i.e.:'INCONS_PARAM':{'To':30,'GRADTZ':0.08,'DEPTH_TO_SURF':100,'DELTAZ':20}

	Returns
	-------
	list
	  T: initial list of temperature
	list
	  P: initial list of pressure
	list
	  depth: range of depth coming from layers
	"""

	layers_info=geomtr.vertical_layers(input_dictionary)
	
	depths=np.arange(min(layers_info['bottom']),max(layers_info['top'])-input_dictionary['INCONS_PARAM']['WL_FROM_TOP_LAYER'],input_dictionary['INCONS_PARAM']['DELTAZ'])
	Pi=1 #[bar]
	To=input_dictionary['INCONS_PARAM']['To']
	depth=0
	T=[]
	P=[]
	for cnt, TVD in enumerate(depths[::-1]):
		if cnt==0:
			z_ref=max(layers_info['top'])+input_dictionary['INCONS_PARAM']['DEPTH_TO_SURF']
		depth=z_ref-TVD
		Ti=input_dictionary['INCONS_PARAM']['GRADTZ']*depth+To
		rho=IAPWS97(T=(Ti+273.15),x=0.0).rho

		if use_boiling:
			P_colum=rho*input_dictionary['INCONS_PARAM']['DELTAZ']*formats.formats_t2['PARAMETERS']['GF'][0]
			Pi+=P_colum/1E5
			P.append(Pi)
		elif use_equation:
			Px=(TVD-b)/m
			P.append(Px)
		
		T.append(Ti)

	depth_i=np.linspace(max(depths),input_dictionary['z_ref'],input_dictionary['INCONS_PARAM']['DELTAZ'])[::-1]
	Ti=np.linspace(input_dictionary['INCONS_PARAM']['T_SURF'],min(T),len(depth_i))
	Pi=np.linspace(1,min(P),len(depth_i))

	depths=np.append(depth_i,depths[::-1])
	T=np.append(Ti,T)
	P=np.append(Pi,P)

	return T, P, depths

def incons(input_dictionary,m,b,use_boiling=True,use_equation=False):
	"""It returns the coordinates X,Y and Z at a desired depth.

	Parameters
	----------
	input_dictionary : dictionary
	  Contains the infomation of the layer under the keyword 'LAYER' and 'z_ref'. Also it contains the keyword 'INCONS_PARAM' with the specified initial conditions, i.e.:'INCONS_PARAM':{'To':30,'GRADTZ':0.08,'DEPTH_TO_SURF':100,'DELTAZ':20}
	m : float
		Pressure slope on a TVD vs P plot
	b : float
		Pressure intercept on a TVD vs P plot
	use_equation : bool
		If true the variables m and b will be use to extrapolate the values to the bottom layer
	use_boiling : bool
		If true the boiling conditions will be use for calculating the bottom conditions		

	Returns
	-------
	file
	  INCON : on model/t2/sources

	Attention
	---------
	It requires an updated ELEME.json

	Note
	----
	Boiling conditions are assumed

	Examples
	--------
	>>> incons(input_dictionary)
	"""
	input_file='../mesh/ELEME.json'
	T,P,depth=initial_conditions(input_dictionary,m,b,use_boiling,use_equation)
	Tfunc=interpolate.interp1d(depth,T)
	Pfunc=interpolate.interp1d(depth,P)
	output_file='../model/t2/sources/INCON'
	string=""
	if os.path.isfile(input_file):
		eleme_df = pd.read_json(input_file).T
		for index, row in eleme_df.iterrows():
			zi=row['Z']
			Ti=Tfunc(zi)
			Pi=Pfunc(zi)*1E5
			string+="%5s%35s\n"%(index,' ')
			string+=" %19.13E %19.13E\n"%(Pi,Ti)
		file=open(output_file,'w')
		file.write(string)
		file.close()
	else:
		sys.exit("The file %s or directory do not exist"%input_file)

def empty_model(structure, current_path='../'):
	"""It erases all the file on the  T2GEORES structure except the input and scripts folders

	Parameters
	----------
	structure : dictionary
	  It is an internal structure dictionary defined on formats.py	
	current_path : str
	  It specifies the top folder of the structure, if the function is executed from scripts current_path is '../'

	Attention
	---------
	It will erase all the output data, mesh and images on the model.


	Examples
	--------
	>>> empty_model(current_path='../')
	"""
	if formats.structure!=None and len(formats.structure):
		for direc in structure:
			empty_model(formats.structure[direc], os.path.join(current_path, direc))
	else:
		for file in os.listdir(current_path):
			if not any(x in current_path for x in ['input','scripts']):
				try:
					os.remove(os.path.join(current_path,file))
				except IsADirectoryError:
					continue

def create_structure(structure,current_path):
	"""It creates the necessary structure for T2GEORES to work

	Parameters
	----------
	structure : dictionary
	  It is an internal structure dictionary defined on formats.py	
	current_path : str
	  It specifies the path where the structure will be constructed

	Note
	----
	A good practice is to create a folder for the whole model

	Examples
	--------
	>>> create_structure(current_path='.')
	"""
	if structure!=None and len(structure):
		for direc in structure:
			create_structure(structure[direc], os.path.join(current_path, direc))
	else:
		os.makedirs(current_path)

def top_layer_incons(input_dictionary,input_mesh_dictionary,cut_off_T=150,cut_off_P=1):
	"""Modifies the incon file base on real data a the top elevation

	Parameters
	----------
	input_dictionary : dictionary
		List the well names, define the INCONS_PARAM and source_txt file.
	input_mesh_dictionary : dictionary
		Specify the polygon border defiding path
	cut_off_T : float
		The heighest value of temperature a the top layer
	cut_off_P : float
		The heighest value of pressure a the top layer 

	Note
	----
	A file called INCON_MOD will be created on t2/sources

	"""

	#If next value true, some plots showing the points where the data is comming from will be plotted
	plot_input_data = True
	plot_points_to_inter = True
	plot_fix_data_points = True
	plot_contour = True

	layers_info=geomtr.vertical_layers(input_dictionary)

	z0=max(layers_info['middle'])

	wells=[]

	for key in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']:
		try:
			for well in input_dictionary[key]:
				wells.append(well)
		except KeyError:
			pass

	eleme_json_file='../mesh/ELEME.json'

	if os.path.isfile(eleme_json_file):
		with open(eleme_json_file) as file:
		  	blocks=json.load(file)
	else:
		sys.exit("The file %s does not exist"%(eleme_json_file))


	wells_corr_json_file='../mesh/well_dict.txt'

	if os.path.isfile(wells_corr_json_file):
		with open(wells_corr_json_file) as file:
		  	wells_json=json.load(file)
	else:
		sys.exit("The file %s does not exist"%(wells_corr_json_file))

	data_T={well:{'T':0,'P':0,'block':'x','extra':[]} for well in wells}

	T0=input_dictionary['INCONS_PARAM']['To']
	P0=input_dictionary['INCONS_PARAM']['Po']

	for well in wells:
		file_name=well+'_MDPT.dat'
		full_file_name=input_dictionary['source_txt']+'PT/'+file_name
		if os.path.isfile(full_file_name):
			well=file_name.split('_')[0]
			data=pd.read_csv(full_file_name,sep=',')
			funcT=interpolate.interp1d(data['MD'],data['T'])
			funcP=interpolate.interp1d(data['MD'],data['P'])

			try:
				if math.isnan(funcT(geomtr.TVD_to_MD(well,z0))) or funcT(geomtr.TVD_to_MD(well,z0))<0:
					if cut_off_T<min(data['T']):
						data_T[well]['T']=cut_off_T
					else:
						data_T[well]['T']=min(data['T'])
				else:
					Tx=funcT(geomtr.TVD_to_MD(well,z0))
					if T0>Tx:
						data_T[well]['T']=T0
					elif cut_off_T<Tx:
						data_T[well]['T']=cut_off_T
					else:
						data_T[well]['T']=Tx

				if math.isnan(funcP(geomtr.TVD_to_MD(well,z0))) or funcP(geomtr.TVD_to_MD(well,z0))<0:
					if cut_off_P<min(data['P']):
						data_T[well]['P']=cut_off_P
					else:
						data_T[well]['P']=min(data['P'])
				else:
					Px=funcP(geomtr.TVD_to_MD(well,z0))
					if P0>Px:
						data_T[well]['P']=P0
					elif cut_off_P<Px:
						data_T[well]['P']=cut_off_P
					else:
						data_T[well]['P']=Px

			except ValueError:
				data_T[well]['T']=cut_off_T
				data_T[well]['P']=cut_off_P
			data_T[well]['block']='A'+wells_json[well][0]

	shape = shapefile.Reader(input_mesh_dictionary['polygon_shape'])

	#first feature of the shapefile
	feature = shape.shapeRecords()[0]
	points = feature.shape.__geo_interface__ 
	polygon=[]
	for n in points:
		for v in points[n]:
			if n=='coordinates':
				polygon.append([v[0],v[1]]) # (GeoJSON format)


	x_real=[]
	y_real=[]
	names=[]
	
	for block in blocks:
		if block[0]=='A':
			x=blocks[block]['X']
			x_real.append(x)
			y=blocks[block]['Y']
			y_real.append(y)
			names.append(block)

	x_inter=[]
	y_inter=[]
	T_real=[]
	P_real=[]
	
	fix_names=[]
	fix_x=[]
	fix_y=[]
	fix_T=[]
	fix_P=[]
	
	for well in data_T:
		xw=blocks[data_T[well]['block']]['X']
		yw=blocks[data_T[well]['block']]['Y']
		for i,x in enumerate(x_real):
			if np.sqrt((xw-x)**2+(yw-y_real[i])**2)<=150 and (x,y_real[i]) not in list(zip(x_inter,y_inter)):
				T_real.append(data_T[well]['T'])
				P_real.append(data_T[well]['P']+0.92)
				y_inter.append(y_real[i])
				x_inter.append(x)

				fix_x.append(x)
				fix_y.append(y_real[i])
				fix_T.append(data_T[well]['T'])
				fix_P.append(data_T[well]['P']+0.92)
				fix_names.append(names[i])

	for i,x in enumerate(x_real):
		if not check_in_out([x,y_real[i]],polygon):
			x_inter.append(x)
			y_inter.append(y_real[i])
			T_real.append(T0)
			P_real.append(P0)

			fix_x.append(x)
			fix_y.append(y_real[i])
			fix_T.append(T0)
			fix_P.append(P0)
			fix_names.append(names[i])

	x_to_inter=[]
	y_to_inter=[]
	blocks_to_inter=[]
	
	for i,x in enumerate(x_real):
		if (x,y_real[i]) not in list(zip(x_inter,y_inter)):
			x_to_inter.append(x)
			y_to_inter.append(y_real[i])
			blocks_to_inter.append(names[i])

	if plot_input_data:
		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		ax.plot(x_inter,y_inter,'og',linestyle='None',ms=2)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.5)
		ax.tick_params(axis='both', which='major', labelsize=10,pad=0.5)
		fig.savefig("../output/initial_conditions/input_points_top_layer.png",dpi=600) 
		plt.show()

	
	if plot_points_to_inter:
		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		ax.plot(x_to_inter,y_to_inter,'ob',linestyle='None',ms=2)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.5)
		ax.tick_params(axis='both', which='major', labelsize=10,pad=0.5)
		fig.savefig("../output/initial_conditions/points_to_interp_top_layer.png",dpi=600) 
		plt.show()


	T_inter=griddata((x_inter,y_inter),T_real,(x_to_inter,y_to_inter),fill_value=T0,method='linear')
	T_inter=np.nan_to_num(T_inter, nan=T0)
	T_inter[T_inter<min(T_real)]=min(T_real)
	
	P_inter=griddata((x_inter,y_inter),P_real,(x_to_inter,y_to_inter),fill_value=P0,method='linear')
	P_inter=np.nan_to_num(P_inter, nan=P0)
	P_inter[P_inter<min(P_real)]=min(P_real)


	data={'blocks':blocks_to_inter+fix_names,
	      'X':np.append(x_to_inter,fix_x),
	      'Y':np.append(y_to_inter,fix_y),
	      'T':np.append(T_inter,fix_T),
	      'P':np.append(P_inter,fix_P)}

	if plot_contour:
		cm_sea_P=sns.color_palette("YlGnBu", as_cmap=True)
		cm_sea_T=sns.color_palette("YlOrRd", as_cmap=True)
		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_title("Temperature distribution, top layer",fontsize=12)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		c=ax.tricontourf(data['X'], data['Y'], data['T'], cmap=cm_sea_T)#,levels=[20,30,40,50,75,100,125,150]
		cbar=fig.colorbar(c,ax=ax)
		cbar.ax.tick_params(labelsize=10)
		cbar.set_label('Temperature [$^\circ$C]',fontsize=11)
		ax.tick_params(axis='both', which='major', labelsize=10,pad=1)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.05)
		fig.savefig("../output/initial_conditions/T_top_layer.png",dpi=600)
		plt.show()

		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_title("Pressure distribution  top layer",fontsize=12)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		pc=ax.tricontourf(data['X'], data['Y'], data['P'],cmap=cm_sea_P)
		cbar=fig.colorbar(pc,ax=ax)
		cbar.ax.tick_params(labelsize=10)
		cbar.set_label('Pressure [bar]',fontsize=11)
		ax.tick_params(axis='both', which='major', labelsize=10,pad=1)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.05)
		fig.savefig("../output/initial_conditions/P_top_layer.png",dpi=600) 
		plt.show()

	string=""
	for i,block in enumerate(data['blocks']):
		string+="%s\n "%block+format(data['P'][i]*1E5,'>19.13E')+" "+format(data['T'][i],'>19.13E')+'\n'

	incon_file='../model/t2/sources/INCON'
	with open(incon_file) as f:
		lines = f.read().splitlines()
	incon=string+'\n'.join(lines[(len(data['T']))*2:-1])
	
	incon_file_out='../model/t2/sources/INCON_MOD'
	incon_out=open(incon_file_out,'w')
	incon_out.write(incon)
	incon_out.close()

def bottom_layer_incons(input_dictionary,input_mesh_dictionary,m,b,use_boiling=True,use_equation=False,cut_off_T=150,cut_off_P=1):
	"""Modifies the incon file base on real data a the bottom or last known elevation

	Parameters
	----------
	input_dictionary : dictionary
		List the well names, define the INCONS_PARAM and source_txt file.
	input_mesh_dictionary : dictionary
		Specify the polygon border defiding path
	cut_off_T : float
		The heighest value of temperature a the bottom layer
	cut_off_P : float
		The heighest value of pressure a the bottom layer
	use_equation : bool
		If true the variables m and b will be use to extrapolate the values to the bottom layer
	use_boiling : bool
		If true the boiling conditions will be use for calculating the bottom conditions

	Note
	----
	A file called INCON_MOD will be modified on t2/sources.Thus, it has to be used after top_layer_incons
	"""

	T, P, depths =initial_conditions(input_dictionary,m,b,use_boiling,use_equation)

	plot_input_data=True
	plot_points_to_inter=True
	plot_fix_data_points=True
	plot_contour=True

	layers_info=geomtr.vertical_layers(input_dictionary)

	z_l=min(layers_info['middle'])

	wells=[]

	for key in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']:
		try:
			for well in input_dictionary[key]:
				wells.append(well)
		except KeyError:
			pass

	eleme_json_file='../mesh/ELEME.json'

	if os.path.isfile(eleme_json_file):
		with open(eleme_json_file) as file:
		  	blocks=json.load(file)
	else:
		sys.exit("The file %s does not exist"%(eleme_json_file))


	wells_corr_json_file='../mesh/well_dict.txt'

	if os.path.isfile(wells_corr_json_file):
		with open(wells_corr_json_file) as file:
		  	wells_json=json.load(file)
	else:
		sys.exit("The file %s does not exist"%(wells_corr_json_file))

	data_T={well:{'T':0,'P':0,'block':'x','extra':[]} for well in wells}

	T0=input_dictionary['INCONS_PARAM']['To']
	P0=input_dictionary['INCONS_PARAM']['Po']


	src='../input/PT'
	src_files = os.listdir(src)
	slopes=[]
	intercepts=[]
	for well in wells:
		file_name=well+'_MDPT.dat'
		full_file_name=src+'/'+file_name
		if os.path.isfile(full_file_name):
			well=file_name.split('_')[0]
			data=pd.read_csv(full_file_name,sep=',')
			data['TVD']=data.apply(lambda x: geomtr.MD_to_TVD(well,x['MD'])[2],axis=1)

			try:
				slope, intercept, r, p, se = linregress(data[data.P > 30]['P'].tolist(), data[data.P > 30]['TVD'].tolist())
				if slope>-15:
					data_T[well]['P']=(z_l-intercept)/(slope)
				else:
					data_T[well]['P']=P[-1]
			except ValueError:
				data_T[well]['P']=P[-1]

			data_T[well]['T']=data['T'].tail(1).values[0]
			data_T[well]['block']=layers_info['name'][-1]+wells_json[well][0]

	shape = shapefile.Reader(input_mesh_dictionary['polygon_shape'])

	#first feature of the shapefile
	feature = shape.shapeRecords()[0]
	points = feature.shape.__geo_interface__ 
	polygon=[]
	for n in points:
		for v in points[n]:
			if n=='coordinates':
				polygon.append([v[0],v[1]]) # (GeoJSON format)

	x_real=[]
	y_real=[]
	names=[]
	
	for block in blocks:
		if block[0]==layers_info['name'][-1]:
			x=blocks[block]['X']
			x_real.append(x)
			y=blocks[block]['Y']
			y_real.append(y)
			names.append(block)

	x_inter=[]
	y_inter=[]
	T_real=[]
	P_real=[]
	
	fix_names=[]
	fix_x=[]
	fix_y=[]
	fix_T=[]
	fix_P=[]
	
	for well in data_T:
		xw=blocks[data_T[well]['block']]['X']
		yw=blocks[data_T[well]['block']]['Y']
		for i,x in enumerate(x_real):
			if np.sqrt((xw-x)**2+(yw-y_real[i])**2)<=150 and (x,y_real[i]) not in list(zip(x_inter,y_inter)):
				T_real.append(data_T[well]['T'])
				P_real.append(data_T[well]['P']+0.92)
				y_inter.append(y_real[i])
				x_inter.append(x)

				fix_x.append(x)
				fix_y.append(y_real[i])
				fix_T.append(data_T[well]['T'])
				fix_P.append(data_T[well]['P']+0.92)
				fix_names.append(names[i])

	fix_dict={'HEAT_SOURCE':{'X':552612,'Y':263398,'R':750,'T':305,'P':P[-1]}}

	for ID in fix_dict:
		xw=fix_dict[ID]['X']
		yw=fix_dict[ID]['Y']
		for i,x in enumerate(x_real):
			if np.sqrt((xw-x)**2+(yw-y_real[i])**2)<=fix_dict[ID]['R'] and (x,y_real[i]) not in list(zip(x_inter,y_inter)) and check_in_out([x,y_real[i]],polygon):
				T_real.append(fix_dict[ID]['T'])
				P_real.append(fix_dict[ID]['P'])
				y_inter.append(y_real[i])
				x_inter.append(x)

				fix_x.append(x)
				fix_y.append(y_real[i])
				fix_T.append(fix_dict[ID]['T'])
				fix_P.append(fix_dict[ID]['P'])
				fix_names.append(names[i])

	for i,x in enumerate(x_real):
		if not check_in_out([x,y_real[i]],polygon):
			x_inter.append(x)
			y_inter.append(y_real[i])
			T_real.append(T[-1])
			P_real.append(P[-1])

			fix_x.append(x)
			fix_y.append(y_real[i])
			fix_T.append(T[-1])
			fix_P.append(P[-1])
			fix_names.append(names[i])

	x_to_inter=[]
	y_to_inter=[]
	blocks_to_inter=[]
	
	for i,x in enumerate(x_real):
		if (x,y_real[i]) not in list(zip(x_inter,y_inter)):
			x_to_inter.append(x)
			y_to_inter.append(y_real[i])
			blocks_to_inter.append(names[i])

	if plot_input_data:
		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		ax.plot(x_inter,y_inter,'og',linestyle='None',ms=2)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.5)
		ax.tick_params(axis='both', which='major', labelsize=10,pad=1)
		fig.savefig("../output/initial_conditions/input_points_bottom_layer.png") 
		plt.show()

	
	if plot_points_to_inter:
		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		ax.plot(x_to_inter,y_to_inter,'ob',linestyle='None',ms=2)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.5)
		ax.tick_params(axis='both', which='major', labelsize=10,pad=1)
		fig.savefig("../output/initial_conditions/points_to_interp_bottom_layer.png") 
		plt.show()


	T_inter=griddata((x_inter,y_inter),T_real,(x_to_inter,y_to_inter),fill_value=T0,method='linear')
	T_inter=np.nan_to_num(T_inter, nan=T0)
	T_inter[T_inter<min(T_real)]=min(T_real)
	
	P_inter=griddata((x_inter,y_inter),P_real,(x_to_inter,y_to_inter),fill_value=P0,method='linear')
	P_inter=np.nan_to_num(P_inter, nan=P0)
	P_inter[P_inter<min(P_real)]=min(P_real)


	data={'blocks':blocks_to_inter+fix_names,
	      'X':np.append(x_to_inter,fix_x),
	      'Y':np.append(y_to_inter,fix_y),
	      'T':np.append(T_inter,fix_T),
	      'P':np.append(P_inter,fix_P)}

	if plot_contour:
		cm_sea_P=sns.color_palette("YlGnBu", as_cmap=True)
		cm_sea_T=sns.color_palette("YlOrRd", as_cmap=True)
		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_title("Temperature distribution, bottom layer",fontsize=12)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		c=ax.tricontourf(data['X'], data['Y'], data['T'],cmap=cm_sea_T)#,levels=[20,30,40,50,75,100,125,150]
		cbar=fig.colorbar(c,ax=ax)
		cbar.ax.tick_params(labelsize=11)
		cbar.set_label('Temperature [$^\circ$C]',fontsize=11)
		ax.tick_params(axis='both', which='major', labelsize=11,pad=1)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.05)
		fig.savefig("../output/initial_conditions/T_bottom_layer.png")
		plt.show()

		fig= plt.figure(figsize=(12, 12), dpi=300)
		ax=fig.add_subplot(111)
		ax.set_title("Pressure distribution, bottom layer",fontsize=12)
		ax.set_xlabel('East [m]')
		ax.set_ylabel('North [m]')
		ax.set_aspect('equal')
		pc=ax.tricontourf(data['X'], data['Y'], data['P'],cmap=cm_sea_P)
		cbar=fig.colorbar(pc,ax=ax)
		cbar.ax.tick_params(labelsize=11)
		cbar.set_label('Pressure [bar]',fontsize=11)
		ax.tick_params(axis='both', which='major', labelsize=11,pad=1)
		ax.plot(x_real,y_real,'ok',linestyle='None',ms=0.05)
		fig.savefig("../output/initial_conditions/P_bottom_layer.png") 
		plt.show()


	string=""
	for i,block in enumerate(data['blocks']):
		string+="%s\n "%block+format(data['P'][i]*1E5,'>19.13E')+" "+format(data['T'][i],'>19.13E')+'\n'
	

	incon_file='../model/t2/sources/INCON_MOD'
	with open(incon_file) as f:
		lines = f.read().splitlines()
	incon='\n'.join(lines[0:(-len(data['T']))*2	])+'\n'+string

	incon_file_out='../model/t2/sources/INCON_MOD'
	incon_out=open(incon_file_out,'w')
	incon_out.write(incon)
	incon_out.close()

def incon_to_steinar(sav = None):
	"""It converts the modified INCON file (INCON_MOD) to steinar input

	Note
	----
	It will write a file called incon_to_steinar on the sources file

	Examples
	--------
	>>> create_structure(current_path='.')
	"""


	if sav == None:
		incon_file='../model/t2/sources/INCON_MOD'
	elif sav == 'init':
		incon_file = '../model/t2/INCON'
	elif sav == 't2.sav':
		incon_file='../model/t2/t2.sav'
	elif sav == 'SAVE1':
		incon_file='../model/t2/SAVE1'

	string=""
	with open(incon_file) as file:
	    for i, line in enumerate(file):
	    	if i!=0:
	    		try:
			    	if line[0].isalpha():
			    		string+=line[0:5]+'    0    0         0.0000\n'
			    	else:
			    		string+=format(float(line[1:20]),'>20.13E')+format(float(line[20:40]),'>20.13E')+'\n'
	    		except ValueError:
	    			break
	    			
	incon_steinar='../mesh/to_steinar/incon_to_steinar'
	file=open(incon_steinar,'w')
	file.write(string)
	file.close()

def check_in_out(point,borders):
	"""It checks if a point is outside or inside of a given area

	Parameters
	----------
	point : array
		x and y position
	borders : array
		Contains the array defining a close polygon
	"""


	polygon=borders
	boolean=False
	cnt=0
	for n in range(len(polygon)):
		if n+1!=len(polygon):
			m,b=plb.polyfit([polygon[n][0],polygon[n+1][0]],[polygon[n][1],polygon[n+1][1]],1)
			val_range=[polygon[n][1],polygon[n+1][1]]
		elif n+1==len(polygon):
			m,b=plb.polyfit([polygon[-1][0],polygon[0][0]],[polygon[-1][1],polygon[0][1]],1)
			val_range=[polygon[-1][1],polygon[0][1]]
		
		x=(point[1]-b)/m
		if point[0]<x and min(val_range)<point[1] and point[1]<max(val_range):
			cnt+=1
	if cnt==1:
		boolean=True

	return boolean



def incon_from_steinar():

	incon_file = '../mesh/to_steinar/incon'

	string=""
	with open(incon_file) as file:
	    for i, line in enumerate(file):
    		try:
		    	if line[0].isalpha():
		    		string+=line[0:5]+'\n'
		    	else:
		    		string+=format(float(line[0:20]),'>20.13E')+format(float(line[20:40]),'>20.13E')+'\n'
    		except ValueError:
    			break
	    			
	incon_steinar='../model/t2/sources/STE_INCON'
	file=open(incon_steinar,'w')
	file.write(string)
	file.close()